//*----------------------------------------------------------------------------
//*         ATMEL Microcontroller Software Support  -  ROUSSET  -
//*----------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*----------------------------------------------------------------------------
//* File Name           : Debug.c
//* Object              : Debug menu
//* Creation            : JPP   16/May/2004
//* Modif               : JPM   16/Nov/2004
//*----------------------------------------------------------------------------

#include <stdio.h>

// Include Standard files
#include "Board.h"
#include "Flash.h"

/*---------------------------- Global Variable ------------------------------*/
char message[80];

//*--------------------------1--------------------------------------------------
//* \fn    AT91F_DBGU_Printk
//* \brief This function is used to send a string through the DBGU channel
//*----------------------------------------------------------------------------
void AT91F_DBGU_Printk(	char *buffer)
{
    while(*buffer != '\0') {
	while (!AT91F_US_TxReady((AT91PS_USART)AT91C_BASE_DBGU));
	AT91F_US_PutChar((AT91PS_USART)AT91C_BASE_DBGU, *buffer++);
    }
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_US_Get
//* \brief Get a Char on USART
//*----------------------------------------------------------------------------
 int AT91F_US_Get( char *val)
{
    if ((AT91F_US_RxReady((AT91PS_USART)AT91C_BASE_DBGU)) == 0) return (false);
    else
    {
	*val= AT91F_US_GetChar((AT91PS_USART)AT91C_BASE_DBGU);
        return (true);
    }
}
//*----------------------------------------------------------------------------
//* \fn    AT91F_DBGU_scanf
//* \brief Get a string on USART and manage Backspace and echo
//*----------------------------------------------------------------------------
void AT91F_DBGU_scanf(char * type,unsigned int * val)
{//* Begin
    unsigned int read = 0;
    char buff[10];
    unsigned int nb_read =0;

    while( (read != 0x0D) & (nb_read != sizeof(buff)) ) {
        //* wait the USART Ready for reception
	 while((AT91C_BASE_DBGU->DBGU_CSR  & AT91C_US_RXRDY) == 0 ) ;
        //* Get a char
	read = AT91C_BASE_DBGU->DBGU_RHR ;
        if(read == 'E') read = 0x0D;
        buff[nb_read]= (char)read;
        //* Manage Backspace
        while((AT91C_BASE_DBGU->DBGU_CSR & AT91C_US_TXRDY) ==0)  {}
        if ((char)read == 0x08) {
            if ( nb_read != 0 ) {
              nb_read--;
              AT91C_BASE_DBGU->DBGU_THR = read;
            }
        }
        else {
          //* echo
          AT91C_BASE_DBGU->DBGU_THR = read;
          nb_read++;
        }
    }
    //* scan the value
 
    sscanf(buff,type,val);
}//* End

//*----------------------------------------------------------------------------
//* \fn   AT91F_DBGU_Flash
//* \brief print address and size
//*----------------------------------------------------------------------------
void AT91F_DBGU_Flash(unsigned int add, unsigned int size)
{
	sprintf( message,"Start 0x%X size 0x%X (%d)\n\r" ,add, size, size);
	AT91F_DBGU_Printk(message);
}
//*----------------------------------------------------------------------------
//* \fn    AT91F_Print_Memory
//* \brief Print a memory area add: xx xx xx xx
//*----------------------------------------------------------------------------
void AT91F_Print_Memory(unsigned int add, unsigned int size)
{
   unsigned int i;
   unsigned int * start_add = (unsigned int *) add;
    AT91F_DBGU_Flash(add,size);
    AT91F_DBGU_Printk("\n\r");
    for (i=0 ; i < size ; i+=4)
    {
       sprintf( message,"At 0x%X: 0x%X 0x%X 0x%X 0x%X\n\r",&start_add[i], start_add[i], start_add[i+1], start_add[i+2], start_add[i+3] );
       AT91F_DBGU_Printk(message);
       AT91F_DBGU_Printk("\n\r");
    }
}
//*----------------------------------------------------------------------------
//* \fn    AT91F_Flash_Lock_info
//* \brief Print Flash NVM bit
//*----------------------------------------------------------------------------
void AT91F_Flash_Lock_info(void)
{
	sprintf(message,"\n\rMC_FSR Value: 0x%X\n\r",AT91C_BASE_MC->MC_FSR);
	AT91F_DBGU_Printk(message);
        AT91F_DBGU_Printk("\n\r");
	if (AT91F_Flash_Lock_Status()!=0)    AT91F_DBGU_Printk("Lock Bit is set\n\r");
        AT91F_DBGU_Printk("\n\r");
	if (AT91F_NVM_Status() !=0)          AT91F_DBGU_Printk("NVM Bit is set\n\r");
        AT91F_DBGU_Printk("\n\r");
	if (AT91F_SET_Security_Status() !=0) AT91F_DBGU_Printk("Security Bit is set\n\r");
  
}

//*----------------------------------------------------------------------------
//* \fn    menu
//* \brief Debug menu
//*----------------------------------------------------------------------------
void menu(void)
{
	unsigned int * start_add;
	int i,size,page;
	char val;
	unsigned int status, buff[FLASH_PAGE_SIZE_LONG*10];
    //* Data initialization
	size = FLASH_PAGE_SIZE_BYTE;
	page = 0;
	start_add = ( unsigned int *) AT91C_IFLASH;
        AT91F_DBGU_Flash (( unsigned int)start_add,size);
	for (i=0; i < FLASH_PAGE_SIZE_LONG*4; i++) { buff[i] = 0xAABBCC00+i;}
    //* Init the flash access
        AT91F_Flash_Init();
        
        AT91F_DBGU_Printk("\n\r");
	AT91F_DBGU_Printk("\n\rMenu:\n\r");
        AT91F_DBGU_Printk("\n\r1) Flash Info\n\r");
        AT91F_DBGU_Printk("\n\r2) Dump Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r3) Write One Page\n\r");
        AT91F_DBGU_Printk("\n\r4) Write Several Pages\n\r");
        AT91F_DBGU_Printk("\n\r5) Erase Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r6) Set Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r7) Clear Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r8) Clear All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\r9) Set All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\rA) Set Security Bit\n\r");
  

    //* Loop menu
	while (1)
	{
        if (AT91F_US_Get(&val))
        switch (val) {

        case '1': //* Flash Info
             AT91F_Flash_Lock_info();
             if (AT91F_Flash_Check_Erase((unsigned int*)AT91C_IFLASH,AT91C_IFLASH_SIZE/4) )  AT91F_DBGU_Printk("Flash is erased\n\r");
             else  AT91F_DBGU_Printk("Flash is not erased !\n\r");
             
        AT91F_DBGU_Printk("\n\r");    
	AT91F_DBGU_Printk("\n\rMenu:\n\r");
        AT91F_DBGU_Printk("\n\r1) Flash Info\n\r");
        AT91F_DBGU_Printk("\n\r2) Dump Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r3) Write One Page\n\r");
        AT91F_DBGU_Printk("\n\r4) Write Several Pages\n\r");
        AT91F_DBGU_Printk("\n\r5) Erase Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r6) Set Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r7) Clear Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r8) Clear All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\r9) Set All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\rA) Set Security Bit\n\r");
        AT91F_DBGU_Printk("\n\rEnter:");
        break;

        case '2': // Dump Flash
             AT91F_Print_Memory((unsigned int)start_add,AT91C_IFLASH_SIZE/4);
             
        AT91F_DBGU_Printk("\n\r");   
	AT91F_DBGU_Printk("\n\rMenu:\n\r");
        AT91F_DBGU_Printk("\n\r1) Flash Info\n\r");
        AT91F_DBGU_Printk("\n\r2) Dump Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r3) Write One Page\n\r");
        AT91F_DBGU_Printk("\n\r4) Write Several Pages\n\r");
        AT91F_DBGU_Printk("\n\r5) Erase Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r6) Set Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r7) Clear Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r8) Clear All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\r9) Set All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\rA) Set Security Bit\n\r");
        AT91F_DBGU_Printk("\n\rEnter:");
        break;

        case '3'://* Write One Page
             AT91F_DBGU_Printk("\n\rPage Number:");
             AT91F_DBGU_scanf("%d",(unsigned int*)&page );
             AT91F_DBGU_Printk("\n\r");
             start_add = ( unsigned int *) AT91C_IFLASH + ( page * FLASH_PAGE_SIZE_LONG);
             AT91F_DBGU_Flash((unsigned int)start_add,FLASH_PAGE_SIZE_LONG);
             if (AT91F_Flash_Write((unsigned int)start_add ,FLASH_PAGE_SIZE_BYTE ,(unsigned int*) &buff) )  AT91F_DBGU_Printk("Flash has been written correctly\n\r");
             else  AT91F_DBGU_Printk("Flash has not been written correctly !\n\r");
             AT91F_DBGU_Printk("\n\r");
             AT91F_Print_Memory((unsigned int)start_add,FLASH_PAGE_SIZE_BYTE/4);
             
        AT91F_DBGU_Printk("\n\r");   
	AT91F_DBGU_Printk("\n\rMenu:\n\r");
        AT91F_DBGU_Printk("\n\r1) Flash Info\n\r");
        AT91F_DBGU_Printk("\n\r2) Dump Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r3) Write One Page\n\r");
        AT91F_DBGU_Printk("\n\r4) Write Several Pages\n\r");
        AT91F_DBGU_Printk("\n\r5) Erase Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r6) Set Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r7) Clear Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r8) Clear All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\r9) Set All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\rA) Set Security Bit\n\r");
        AT91F_DBGU_Printk("\n\rEnter:");
        break;            
        
        case '4'://* Write Several Pages
             AT91F_DBGU_Printk("\n\rAddress (Hex value):");
            AT91F_DBGU_scanf("%X",(unsigned int*)&size );
 
             start_add = (unsigned int *)size;
             AT91F_DBGU_Printk("\n\rSize in byte (Dec value):");
             AT91F_DBGU_scanf("%d",(unsigned int*)&size );
 
             AT91F_DBGU_Flash((unsigned int)start_add,size);
             AT91F_DBGU_Printk("\n\r");   
             if (AT91F_Flash_Write_all((unsigned int)start_add ,size ,(unsigned int*) &buff) )  AT91F_DBGU_Printk("Flash has been written correctly\n\r");
             else  AT91F_DBGU_Printk("Flash has not been written correctly !\n\r");
             AT91F_Print_Memory((unsigned int)start_add,(size/4));
             AT91F_DBGU_Printk("\n\r");
                            
        AT91F_DBGU_Printk("\n\r");   
	AT91F_DBGU_Printk("\n\rMenu:\n\r");
        AT91F_DBGU_Printk("\n\r1) Flash Info\n\r");
        AT91F_DBGU_Printk("\n\r2) Dump Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r3) Write One Page\n\r");
        AT91F_DBGU_Printk("\n\r4) Write Several Pages\n\r");
        AT91F_DBGU_Printk("\n\r5) Erase Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r6) Set Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r7) Clear Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r8) Clear All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\r9) Set All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\rA) Set Security Bit\n\r");
        AT91F_DBGU_Printk("\n\rEnter:");
        break;          
        
        case '5'://* Erase All Flash
              if (AT91F_Flash_Erase_All()) AT91F_DBGU_Printk("\n\rFlash cmd OK\n\r");;
              if (AT91F_Flash_Check_Erase((unsigned int*)AT91C_IFLASH,AT91C_IFLASH_SIZE) )  AT91F_DBGU_Printk("The entire flash has been erased\n\r");
              else  AT91F_DBGU_Printk("The entire flash has not been erased! Check lock bits\n\r");
              
                                
        AT91F_DBGU_Printk("\n\r");   
	AT91F_DBGU_Printk("\n\rMenu:\n\r");
        AT91F_DBGU_Printk("\n\r1) Flash Info\n\r");
        AT91F_DBGU_Printk("\n\r2) Dump Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r3) Write One Page\n\r");
        AT91F_DBGU_Printk("\n\r4) Write Several Pages\n\r");
        AT91F_DBGU_Printk("\n\r5) Erase Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r6) Set Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r7) Clear Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r8) Clear All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\r9) Set All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\rA) Set Security Bit\n\r");
        AT91F_DBGU_Printk("\n\rEnter:");
        break;                   

        case '6'://* Set Lock Bit
             AT91F_DBGU_Printk("\n\rPage Number:");
             AT91F_DBGU_scanf("%d",(unsigned int*)&page );
             sprintf( message,"\n\rSET lock %d 0x%X\n\r",page,AT91F_Flash_Lock (page));
             AT91F_DBGU_Printk(message);
                                              
        AT91F_DBGU_Printk("\n\r");   
	AT91F_DBGU_Printk("\n\rMenu:\n\r");
        AT91F_DBGU_Printk("\n\r1) Flash Info\n\r");
        AT91F_DBGU_Printk("\n\r2) Dump Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r3) Write One Page\n\r");
        AT91F_DBGU_Printk("\n\r4) Write Several Pages\n\r");
        AT91F_DBGU_Printk("\n\r5) Erase Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r6) Set Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r7) Clear Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r8) Clear All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\r9) Set All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\rA) Set Security Bit\n\r");
        AT91F_DBGU_Printk("\n\rEnter:");
        break;               
 
        case '7'://* Clear Lock bit
             AT91F_DBGU_Printk("\n\rPage Number:");
             AT91F_DBGU_scanf("%d",(unsigned int*)&page );
             sprintf( message,"\n\rCLEAR lock %d 0x%X\n\r",page,AT91F_Flash_Unlock (page));
             AT91F_DBGU_Printk(message);
                                                               
        AT91F_DBGU_Printk("\n\r");   
	AT91F_DBGU_Printk("\n\rMenu:\n\r");
        AT91F_DBGU_Printk("\n\r1) Flash Info\n\r");
        AT91F_DBGU_Printk("\n\r2) Dump Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r3) Write One Page\n\r");
        AT91F_DBGU_Printk("\n\r4) Write Several Pages\n\r");
        AT91F_DBGU_Printk("\n\r5) Erase Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r6) Set Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r7) Clear Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r8) Clear All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\r9) Set All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\rA) Set Security Bit\n\r");
        AT91F_DBGU_Printk("\n\rEnter:");
        break;               

        case '8'://* Clear All Lock Bits
             AT91F_DBGU_Printk("\n\r");
             for (i=0;i<FLASH_PAGE_NB;i+=FLASH_PAGE_LOCK)
             {
                 status =AT91F_Flash_Unlock (i);
                 sprintf(message,"Clear page %d 0x%X\n\r",i,status);
                 AT91F_DBGU_Printk(message);
             }
                                                            
        AT91F_DBGU_Printk("\n\r");   
	AT91F_DBGU_Printk("\n\rMenu:\n\r");
        AT91F_DBGU_Printk("\n\r1) Flash Info\n\r");
        AT91F_DBGU_Printk("\n\r2) Dump Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r3) Write One Page\n\r");
        AT91F_DBGU_Printk("\n\r4) Write Several Pages\n\r");
        AT91F_DBGU_Printk("\n\r5) Erase Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r6) Set Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r7) Clear Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r8) Clear All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\r9) Set All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\rA) Set Security Bit\n\r");
        AT91F_DBGU_Printk("\n\rEnter:");
        break;               

        case '9'://* Set All Lock Bits
              AT91F_DBGU_Printk("\n\r");
              for (i=0;i<FLASH_PAGE_NB;i+=FLASH_PAGE_LOCK)
              {
                   sprintf(message,"SET page %d 0x%X\n\r",i,AT91F_Flash_Lock (i));
                   AT91F_DBGU_Printk(message);
              }
                                                                 
        AT91F_DBGU_Printk("\n\r");   
	AT91F_DBGU_Printk("\n\rMenu:\n\r");
        AT91F_DBGU_Printk("\n\r1) Flash Info\n\r");
        AT91F_DBGU_Printk("\n\r2) Dump Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r3) Write One Page\n\r");
        AT91F_DBGU_Printk("\n\r4) Write Several Pages\n\r");
        AT91F_DBGU_Printk("\n\r5) Erase Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r6) Set Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r7) Clear Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r8) Clear All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\r9) Set All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\rA) Set Security Bit\n\r");
        AT91F_DBGU_Printk("\n\rEnter:");
        break;               
  
        case 'A'://* Set Security Bit
             sprintf(message,"\r\nSET Security Bit\n\r", AT91F_SET_Security());
             AT91F_DBGU_Printk(message);
             
                                                    
        AT91F_DBGU_Printk("\n\r");   
	AT91F_DBGU_Printk("\n\rMenu:\n\r");
        AT91F_DBGU_Printk("\n\r1) Flash Info\n\r");
        AT91F_DBGU_Printk("\n\r2) Dump Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r3) Write One Page\n\r");
        AT91F_DBGU_Printk("\n\r4) Write Several Pages\n\r");
        AT91F_DBGU_Printk("\n\r5) Erase Entire Flash\n\r");
        AT91F_DBGU_Printk("\n\r6) Set Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r7) Clear Lock Bit\n\r");
        AT91F_DBGU_Printk("\n\r8) Clear All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\r9) Set All Lock Bits\n\r");
        AT91F_DBGU_Printk("\n\rA) Set Security Bit\n\r");
        AT91F_DBGU_Printk("\n\rEnter:");
        break;               

                default:
                break;
		}// End Switch
	}// End while

}


