/*-----------------------------------------------------------------------------
*         ATMEL Microcontroller Software Support  -  ROUSSET  -
*------------------------------------------------------------------------------
* The software is delivered "AS IS" without warranty or condition of any
* kind, either express, implied or statutory. This includes without
* limitation any warranty or condition with respect to merchantability or
* fitness for any particular purpose, or against the infringements of
* intellectual property rights of others.
*------------------------------------------------------------------------------
* File Name           : GLCD-TempSensor.c
* Object              : main application written in C.
*                     : Graphics LCD Demo Example
* Version | mm | dd | yy | author :
*  1.0      11   17   06    PFi   : Creation
*
*------------------------------------------------------------------------------
* 1)Description:
*
* This Demo uses a I2C Graphic LCD module with the Philips PCF8558 controller
* connnected on the TWI port.
*
* ------| AT91 / IO  | GLCD / IO
* CLOCK | TWCK / PA3 |  5
* DATA  | TWD / PA4  |  1
*
* * WARNINGS ** :
* - No pull ups are mounted on the board on TWD and TWCK. 2.2K Pull up used
* - By default IAR compiler use the Tiny printf and scanf library format.
*   To be able to make print of float variables, you must set the library
*   format to large.
*
*  For further details check the following Application note
*  "AT91-AN01: Using the Two-wire interface (TWI) in
*  Master Mode on AT91SAM Microcontrollers."
*  www.atmel.com/dyn/products/app_notes.asp?family_id=605
*
* - By default IAR compiler use the Tiny printf and scanf library format.
*   To be able to make print of float variables, you must set the library
*   format to large.
*
* Remarks:
*
* This example runs in polling mode.and makes no use of IRQs. To have further
* details and implementation example about using IRQs, refer to any Interrupt
* project examples provided in Getting Started available on
* www.atmel.com/dyn/products/app_notes.asp?family_id=605*
-----------------------------------------------------------------------------*/

// Include Standard LIB  files
#include "Board.h"
#include "common\lib_twi.h"
#include "glcd_batron\glcd_i2c_batron.h"

#define TWI_BUS_CLOCK   400000 /* in Hz*/

#define PIO_INTERRUPT_LEVEL 6

// Global Variables
volatile unsigned int Next = 0 ;   /* Hold the next demo to call */

typedef void (*ptFunction)(void);  /* Pointer to a function */

ptFunction ptFunctionTable[3];     /* Array of Pointer to functions */

int IndexPtFunction = 1;

/* External functions */
extern void AT91F_WaitSecond (unsigned int seconde);
extern void AT91F_WaitMiliSecond (unsigned int MiliSeconds);

/* Available in demo.c */
extern void AT91F_Demo1(void);
extern void AT91F_Demo2(void);
extern void AT91F_Demo3(void);


/*-----------------------------------------------------------------------------
* Function Name       : void AT91F_Pio_C_Irq_Handler ( void )
* Object              : Irq C Handler
* Input Parameters    : none
* Output Parameters   : none
-----------------------------------------------------------------------------*/
void AT91F_Pio_C_Irq_Handler ( void )
{
  unsigned int dummy, Sw4IrqState ;

  /* read the interrupt status reg of the PIO to acknoledge the IRQ */
  dummy = AT91C_BASE_PIOA->PIO_ISR;
  dummy = dummy;

  Sw4IrqState = ((AT91C_BASE_PIOA->PIO_PDSR) & SW4_MASK) >> 24 ;

  if (!Sw4IrqState ) /* SW4 push button IRQ */
  {
    /* Stay here a little bit for debouncing */
    AT91F_WaitMiliSecond(150 ) ;

    IndexPtFunction++;

    if ( IndexPtFunction == 3)
      IndexPtFunction= 1;
  }

}


/*-----------------------------------------------------------------------------
* Function Name       : main
* Object              : Main function
* Input Parameters    : none
* Output Parameters   : TRUE
-----------------------------------------------------------------------------*/
int main( void )
//* Begin
{
    ptFunctionTable[0]= &AT91F_Demo1;
    ptFunctionTable[1]= &AT91F_Demo2;
    ptFunctionTable[2]= &AT91F_Demo3;

    /* Pointer to the AIC */
    AT91PS_AIC     pAic;

    /* Load System pAic Base address */
    pAic = AT91C_BASE_AIC;

    /* Enable User Reset and set its minimal assertion to 960 us */
    AT91C_BASE_RSTC->RSTC_RMR = AT91C_RSTC_URSTEN | (0x4<<8) | (unsigned int)(0xA5<<24);

    /* TWI intialization @ TWI_BUS_CLOCK for all TWI devices connected on the bus */
    AT91F_TWI_Open(TWI_BUS_CLOCK);

    /* First, enable the clock of the PIOA */
    AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_PIOA ) ;

    /* Configure SW4 push button connected on the PIO line
    PA14 to switch between demos. */
    AT91F_PIO_CfgInput(AT91C_BASE_PIOA, SW4_MASK);

    /* configure the PIO line corresponding to SW4
    to generate an IRQ on inputchange level */
    AT91F_AIC_ConfigureIt (pAic,
                           AT91C_ID_PIOA,
                           PIO_INTERRUPT_LEVEL,
                           AT91C_AIC_SRCTYPE_HIGH_LEVEL,
                           AT91F_Pio_C_Irq_Handler);

    /* Clear the LCD and put it in normal mode */
    AT91F_Glcd_ClearLcd();
    AT91F_Glcd_VideoMode(Normal);

    /* Print Demo Header */
    /* Print that string on the LCD. Max 19 characters per line/string.*/
    AT91F_Glcd_PutString("###################",LCD_PAGE_0,0);
    AT91F_Glcd_PutString("###################",LCD_PAGE_1,0);
    AT91F_Glcd_PutString(" Graphic LCD Demo ",LCD_PAGE_2,0);
    AT91F_Glcd_PutString("###################",LCD_PAGE_3,0);
    AT91F_Glcd_PutString("###################",LCD_PAGE_4,0);

    AT91F_WaitSecond(2);

    // Clear the LCD
    AT91F_Glcd_ClearLcd();
    AT91F_Glcd_VideoMode(Normal);

    AT91F_Demo1();

    // Clear the LCD
    AT91F_Glcd_ClearLcd();


    // Enable the IRQ on that pin at  PIO level
    AT91F_PIO_InterruptEnable(AT91C_BASE_PIOA,SW4_MASK);

    //* set the interrupt by software
    AT91F_AIC_EnableIt (pAic, AT91C_ID_PIOA);

    // Call the Demos. IndexPtFunction is changed by IRQ on SW4
      while (1)
        ptFunctionTable[IndexPtFunction]();


}//* End

