/*-----------------------------------------------------------------------------
*         ATMEL Microcontroller Software Support  -  ROUSSET  -
*------------------------------------------------------------------------------
* The software is delivered "AS IS" without warranty or condition of any
* kind, either express, implied or statutory. This includes without
* limitation any warranty or condition with respect to merchantability or
* fitness for any particular purpose, or against the infringements of
* intellectual property rights of others.
*------------------------------------------------------------------------------
* File Name           : lm75.h
* Object              : LM75 Temperature Sensor definition
* Creation            :
* 1.0 19/Sept/06 PFi   : Creation
*----------------------------------------------------------------------------*/
#ifndef lm75_h
#define lm75_h
#include <stdio.h>

/* Internal addr: "1001" + A0=A1=A2= +VDD */
#define AT91C_LM75_I2C_ADDRESS (0x4F<<16)

/* Temperature data can be read from the Temperature, TOS
Set Point, and THYST Set Point registers; and written to the
TOS Set Point, and THYST Set Point registers. Temperature
data is represented by a 9-bit, twos complement word with
an LSB (Least Significant Bit) equal to 0.5C */

/* Temperature below are given in degree celsius */
#define TEMP_PLUS_125  0xFA  // +125C
#define TEMP_PLUS_25   0x32  // +25C
#define TEMP_PLUS_0_5  0x1   // +0.5C
#define TEMP_PLUS_0    0x0   // +0C
#define TEMP_MINUS_0_5  0x1   // -0.5C
#define TEMP_MINUS_25   0x32  // -25C
#define TEMP_MINUS_55  0xFA  // -55C

/* Register definition */

/*****************
Pointer register *
******************
P7 | P6 | P5 | P4 | P3 | P2 | P1   |    P0 |
00 | 00 | 00 | 00 | 00 | 00 |Register Select

P0-P1: Register Select:
P1 | P0 : Register
0 |  0 Temperature (Read only) (Power-up default)
0 |  1 Configuration (Read/Write)
1 |  0 THYST (Read/Write)
1 |  1 TOS (Read/Write) */
#define TEMP_REG  0x0
#define CONFIG_REG 0x1
#define THYST_REG 0x2
#define TOS_REG 0x3

/*********************
TEMPERATURE REGISTER *
**********************
(Read Only):
D15 | D14 | D13 | D12 | D11 | D10 | D9  | D8 | D7 | D6| D5| D4| D3| D2| D1| D0 |
MSB |Bit7 |Bit6 |Bit5 |Bit4 |Bit3 |Bit2 |Bit1|LSB | X | X | X | X | X | X |  X |
D0D6: Undefined
D7D15: Temperature Data. One LSB = 0.5C. Twos
complement format.*/

#define TEMP_REG_MASK 0xFF80

/***********************
CONFIGURATION REGISTER *
************************
(Read/Write):
D7 | D6| D5| D4  |   D3 | D2 | D1     | D0       |
 0 | 0 | 0 |Fault Queue |O.S |Cmp/Int | Shutdown |
Power up default is with all bits 0 (zero).*/
#define SHUTDOWN_MODE 0x1
#define INT_MODE 0x2  // Default Mode = Comp Mode
#define POL_HIGH 0x4  // Default Pol = Low Polarity

/*Number of faults necessary to detect before setting O.S. output*/
#define NUM_FAULT_2 0x8
#define NUM_FAULT_4 0x10
#define NUM_FAULT_6 0x18

/***********************
THYST AND TOS REGISTER *
************************
(Read/Write):
D15|D14 | D13| D12| D11| D10| D9 | D8 |D7 |D6|D5 |D4 |D3 |D2 |D1 |D0 |
MSB|Bit7|Bit6|Bit5|Bit4|Bit3|Bit2|Bit1|LSB| X| X | X | X | X | X | X |

D7D15: THYST Or TOS Trip Temperature Data. Power up
default is TOS = 80C, THYST = 75C.*/

extern char AT91F_LM75_ReadConfigurationRegister(void);
extern int AT91F_LM75_SetConfigurationRegister(char ConfigRegValue);
extern float AT91F_LM75_ReadThystRegister(void);
extern int AT91F_LM75_SetThysRegister(short ThystRegValue);
extern float AT91F_LM75_ReadTosRegister(void);
extern int AT91F_LM75_SetTosRegister(short TosRegValue);
extern float AT91F_LM75_ReadTempRegister(void);

extern void AT91F_DBGU_Init(void);
extern unsigned int AT91F_DBGU_Get( char *val);
extern void AT91F_DBGU_scanf(char * type,unsigned int * val);
extern void AT91F_DBGU_Printk(char *);

extern void Trace_Toggel_LED (unsigned int Led);

#endif
